CREATE OR REPLACE FUNCTION izrac_kon_cif_sifre_trenera (prezime_trenera varchar, ime_trenera varchar, brojac varchar) RETURNS smallint AS
$body$

--
-- Funkcija 'izrac_kon_cif_sifre_trenera' od prosledjenih parametar racuna kontrolnu cifru
--   - prezime_trenera: Dovoljno je proslediti samo pocetno slovo
--   - ime_trenera: Dovoljno je proslediti samo pocetno slovo
--   - brojac: Redosled
--   - Rezultat: kontrolna cifra
--

DECLARE

  i     SMALLINT;
  s1    CHAR(1);
  s2    CHAR(1);
  saldo SMALLINT;

BEGIN

  s1    = SUBSTRING(prezime_trenera FROM 1 FOR 1);
  s2    = SUBSTRING(ime_trenera FROM 1 FOR 1);

  -- sifra trenera je ispravnog formata te se pristupa racunanju kontrolne cifre
  saldo = ASCII(s1) + ASCII(s2);
  FOR i IN 1 .. 2 LOOP
    saldo = saldo + CAST(SUBSTRING(brojac from i for 1) AS SMALLINT);
  END LOOP;

  RETURN CAST(MOD(saldo, 10) AS CHAR(1));

END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




CREATE OR REPLACE FUNCTION proveri_sifru_trenera (sifra_trenera varchar) RETURNS smallint AS
$body$

--
-- Funkcija 'proveri_sifru_trenera' proverava ispravnost prosledjene sifre_trenera
--   - sifra_trenera: u formatu SSCCK
--       S = veliko slovo
--       C = cifra
--       K = kontrolna cifra
--   - Rezultat:
--       1 = ako je sifra trenera ispravna,
--       0 = ako sifra trenera nije ispravna.
--

DECLARE

  cifre CHAR(3);
  i     SMALLINT;
  s1    CHAR(1);
  s2    CHAR(1);
  saldo SMALLINT;

BEGIN

  s1    = SUBSTRING(sifra_trenera FROM 1 FOR 1);
  s2    = SUBSTRING(sifra_trenera FROM 2 FOR 1);
  cifre = SUBSTRING(sifra_trenera FROM 3 FOR 3);

  IF    (NOT s1 BETWEEN 'A' AND 'Z')
     OR (NOT s2 BETWEEN 'A' AND 'Z')
     OR (cifre !~ '[[:digit:]]{3}')
  THEN

    -- sifra trenera je ocigledno losa
    RETURN 0;

  ELSE

    -- sifra trenera je ispravnog formata te se pristupa racunanju kontrolne cifre
    saldo = ASCII(s1) + ASCII(s2);
    FOR i IN 1 .. 2 LOOP
      saldo = saldo + CAST(SUBSTRING(cifre from i for 1) AS SMALLINT);
    END LOOP;

    -- proveri kontrolnu cifru
    IF CAST(SUBSTRING(cifre FROM 3 FOR 1) AS SMALLINT) = MOD(saldo, 10) THEN
      RETURN 1;
    ELSE
      RETURN 0;
    END IF;

  END IF;

END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




ALTER TABLE Treneri
  ADD CONSTRAINT ch_tre_sifra_trenera
  CHECK (proveri_sifru_trenera(sifra_trenera) = 1);




CREATE FUNCTION "tr_tre_postavi_sifru_trenera" () RETURNS trigger AS
$body$
DECLARE

  b      INTEGER;
  brojac CHAR(3);
  s1     CHAR(1);
  s2     CHAR(1);

BEGIN

  s1 = UPPER(SUBSTRING(NEW.prezime_trenera FROM 1 FOR 1));
  s2 = UPPER(SUBSTRING(NEW.ime_trenera FROM 1 FOR 1));
  
  SELECT INTO b
         COUNT(*)
    FROM Treneri
   WHERE sifra_trenera LIKE s1 || s2 || '%';
  b = COALESCE(b, 0) + 1;
  brojac = LPAD(CAST(b AS CHAR(2)), 2, '0');
         
  NEW.sifra_trenera = s1 || s2 || brojac || izrac_kon_cif_sifre_trenera(s1, s2, brojac);

  RETURN NEW;
  
END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




CREATE TRIGGER tre_postavi_sifru_trenera BEFORE INSERT 
  ON Treneri FOR EACH ROW 
  EXECUTE PROCEDURE tr_tre_postavi_sifru_trenera();




CREATE OR REPLACE FUNCTION izrac_kon_cif_sifre_deteta (prezime_deteta varchar, ime_deteta varchar, godiste_deteta varchar, brojac varchar) RETURNS smallint AS
$body$

--
-- Funkcija 'izrac_kon_cif_sifre_deteta' od prosledjenih parametar racuna kontrolnu cifru
--   - prezime_deteta: Dovoljno je proslediti samo pocetno slovo
--   - ime_deteta: Dovoljno je proslediti samo pocetno slovo
--   - godiste_deteta
--   - brojac: Redosled
--   - Rezultat: kontrolna cifra
--

DECLARE

  i     SMALLINT;
  s1    CHAR(1);
  s2    CHAR(1);
  saldo SMALLINT;
  cifre CHAR(7);

BEGIN

  s1    = SUBSTRING(prezime_deteta FROM 1 FOR 1);
  s2    = SUBSTRING(ime_deteta FROM 1 FOR 1);
  cifre = godiste_deteta || brojac;

  saldo = ASCII(s1) + ASCII(s2);
  FOR i IN 1 .. 7 LOOP
    saldo = saldo + CAST(SUBSTRING(cifre from i for 1) AS SMALLINT);
  END LOOP;

  RETURN CAST(MOD(saldo, 10) AS CHAR(1));

END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




CREATE OR REPLACE FUNCTION proveri_sifru_deteta (sifra_deteta varchar) RETURNS smallint AS
$body$

--
-- Funkcija 'proveri_sifru_deteta' proverava ispravnost prosledjene sifre deteta
--   - sifra_deteta: u formatu SSCCCCCCCK
--       S = veliko slovo
--       C = cifra
--       K = kontrolna cifra
--   - Rezultat:
--       1 = ako je sifra deteta ispravna,
--       0 = ako sifra deteta nije ispravna.
--

DECLARE

  cifre CHAR(8);
  i     SMALLINT;
  s1    CHAR(1);
  s2    CHAR(1);
  saldo SMALLINT;

BEGIN

  s1    = SUBSTRING(sifra_deteta FROM 1 FOR 1);
  s2    = SUBSTRING(sifra_deteta FROM 2 FOR 1);
  cifre = SUBSTRING(sifra_deteta FROM 3 FOR 8);

  IF    (NOT s1 BETWEEN 'A' AND 'Z')
     OR (NOT s2 BETWEEN 'A' AND 'Z')
     OR (cifre !~ '[[:digit:]]{8}')
  THEN

    -- sifra deteta je ocigledno losa
    RETURN 0;

  ELSE

    -- sifra deteta je ispravnog formata te se pristupa racunanju kontrolne cifre
    saldo = ASCII(s1) + ASCII(s2);
    FOR i IN 1 .. 7 LOOP
      saldo = saldo + CAST(SUBSTRING(cifre from i for 1) AS SMALLINT);
    END LOOP;

    -- proveri kontrolnu cifru
    IF CAST(SUBSTRING(cifre FROM 8 FOR 1) AS SMALLINT) = MOD(saldo, 10) THEN
      RETURN 1;
    ELSE
      RETURN 0;
    END IF;

  END IF;

END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




ALTER TABLE Deca
  ADD CONSTRAINT ch_dec_sifra_deteta
  CHECK (proveri_sifru_deteta(sifra_deteta) = 1);




CREATE FUNCTION "tr_dec_postavi_sifru_deteta" () RETURNS trigger AS
$body$
DECLARE

  b      INTEGER;
  brojac CHAR(3);
  g      CHAR(4);
  s1     CHAR(1);
  s2     CHAR(1);

BEGIN

  s1 = UPPER(SUBSTRING(NEW.prezime_deteta FROM 1 FOR 1));
  s2 = UPPER(SUBSTRING(NEW.ime_deteta FROM 1 FOR 1));
  g  = CAST(NEW.godiste_deteta AS CHAR(4));
  
  SELECT INTO b
         COUNT(*)
    FROM Deca
   WHERE sifra_deteta LIKE s1 || s2 || g || '%';
  b = COALESCE(b, 0) + 1;
  brojac = LPAD(CAST(b AS CHAR(3)), 3, '0');
         
  NEW.sifra_deteta = s1 || s2 || g || brojac || izrac_kon_cif_sifre_deteta(s1, s2, g, brojac);

  RETURN NEW;
  
END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;




CREATE TRIGGER dec_postavi_sifru_deteta BEFORE INSERT 
  ON Deca FOR EACH ROW 
  EXECUTE PROCEDURE tr_dec_postavi_sifru_deteta();




CREATE FUNCTION f_uta_jedan_tim_u_jednom_kolu () RETURNS trigger AS
$body$
--
-- zamena za CREATE ASSERTION
--

BEGIN

  IF EXISTS (SELECT redni_broj_sezone,
                    redni_broj_kola,
                    skracenica_divizije,
                    takmicarski_broj_domaceg_tima
               FROM Utakmice
             INTERSECT
             SELECT redni_broj_sezone,
                    redni_broj_kola,
                    skracenica_divizije,
                    takmicarski_broj_gostujuceg_tima
               FROM Utakmice)
  THEN

    RAISE EXCEPTION 'Tim % i/ili tim % vec igraju u kolu % divizije %!',
                    NEW.takmicarski_broj_domaceg_tima,
                    NEW.takmicarski_broj_gostujuceg_tima,
                    NEW.redni_broj_kola,
                    NEW.skracenica_divizije;

  END IF;

  RETURN NEW;
  
END;
$body$
LANGUAGE 'plpgsql' VOLATILE CALLED ON NULL INPUT SECURITY INVOKER;

CREATE TRIGGER tr_uta_jedan_tim_u_jednom_kolu AFTER INSERT OR UPDATE 
  ON utakmice FOR EACH ROW 
  EXECUTE PROCEDURE f_uta_jedan_tim_u_jednom_kolu();
  
  
  
  
